<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Form\Extension;

use Dedi\SyliusDpdPickUpPlugin\Entity\OrderInterface;
use Dedi\SyliusDpdPickUpPlugin\Exception\MissingPudoException;
use Dedi\SyliusDpdPickUpPlugin\Model\Pudo;
use Dedi\SyliusDpdPickUpPlugin\Service\PudoSelector;
use Sylius\Bundle\CoreBundle\Form\Type\Checkout\SelectShippingType;
use Symfony\Component\Form\AbstractTypeExtension;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Contracts\Translation\TranslatorInterface;
use Webmozart\Assert\Assert;

class SelectShippingTypeExtension extends AbstractTypeExtension
{
    private PudoSelector $pudoSelector;
    private TranslatorInterface $translator;

    public function __construct(
        PudoSelector $pudoSelector,
        TranslatorInterface $translator
    ) {
        $this->pudoSelector = $pudoSelector;
        $this->translator = $translator;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('dpdPickUpPoint', HiddenType::class, [
                'required' => true,
                'mapped' => false,
            ])
        ;

        $builder->addEventListener(FormEvents::POST_SUBMIT, function (FormEvent $event): void {
            $form = $event->getForm();
            $order = $event->getData();

            Assert::isInstanceOf($order, OrderInterface::class);

            $pudo = null;
            if ($form->get('dpdPickUpPoint')->getData()) {
                $data = json_decode($form->get('dpdPickUpPoint')->getData(), true);
                $pudo = Pudo::fromArray($data);
            }

            try {
                $this->pudoSelector->__invoke($order, $pudo);
            } catch (MissingPudoException $e) {
                $form->addError(new FormError(
                    $this->translator->trans('dedi_dpd_pick_up.missing_pudo', [], 'validators')
                ));
            }
        });
    }

    public static function getExtendedTypes(): iterable
    {
        return [
            SelectShippingType::class,
        ];
    }
}
